package buildme

import (
	"context"
	"io"
	"os"
	"path"

	"dev.hexasoftware.com/hxs/prettylog"

	"github.com/docker/docker/api/types"
	"github.com/docker/docker/api/types/container"
	docker "github.com/docker/docker/client"
)

type ImageCtx struct {
	imageName string // Starting image
	//conID     string
	tmpImgID string
	//containerName string // Temporary
	cli *docker.Client
}

func ImageCli(imageName string) (*ImageCtx, error) {
	cli, err := docker.NewEnvClient()
	if err != nil {
		return nil, err
	}
	return &ImageCtx{imageName, imageName, cli}, nil
}

func (c *ImageCtx) Run(command []string) error {
	log := prettylog.New("RUN in IMAGE:")
	log.Println("RUN", command)

	ctx := context.Background()

	if c.tmpImgID == "" {
		c.tmpImgID = c.imageName
	}

	conConfig := &container.Config{Image: c.tmpImgID, Cmd: command}
	ncon, err := c.cli.ContainerCreate(ctx, conConfig, nil, nil, "")
	if err != nil {
		return err
	}
	defer c.cli.ContainerRemove(ctx, ncon.ID, types.ContainerRemoveOptions{})
	//c.conID = ncon.ID

	err = c.cli.ContainerStart(ctx, ncon.ID, types.ContainerStartOptions{})
	if err != nil {
		return err
	}
	retCode, err := c.cli.ContainerWait(ctx, ncon.ID)
	if err != nil {
		return err
	}
	_ = retCode
	/*if retCode != 0 {
		return errors.New("Container did not complete successfully")
	}*/

	imgRes, err := c.cli.ContainerCommit(ctx, ncon.ID, types.ContainerCommitOptions{Config: conConfig})
	c.tmpImgID = imgRes.ID

	c.cli.ImageTag(ctx, imgRes.ID, c.imageName+":building")
	return nil
}

func (c *ImageCtx) Get(src string, dst string) error {
	log := prettylog.New("COPY in IMAGE:")
	log.Println("GET", src, dst)
	ctx := context.Background()

	conConfig := &container.Config{Image: c.tmpImgID, Cmd: []string{""}}
	// Create container
	log.Println("ContainerCreate from: ", c.tmpImgID)
	ncon, err := c.cli.ContainerCreate(ctx, conConfig, nil, nil, "")
	if err != nil {
		return err
	}
	defer c.cli.ContainerRemove(ctx, ncon.ID, types.ContainerRemoveOptions{})

	rd, p, err := c.cli.CopyFromContainer(ctx, ncon.ID, src)
	if err != nil {
		return err
	}
	defer rd.Close()
	log.Println("Copy:", p)

	dstStat, err := os.Stat(dst)
	if err != nil {
		return err
	}
	if dstStat.IsDir() {
		dst = path.Join(dst, p.Name)
		log.Println("Desitnation is dir, adding path", dst)
	}

	f, err := os.OpenFile(dst, os.O_CREATE|os.O_WRONLY, p.Mode)
	if err != nil {
		return err
	}
	io.Copy(f, rd)
	f.Close()

	return nil
}
