package git

import (
	"buildme/utils/tarutils"
	"io/ioutil"
	"os"
	"path/filepath"

	"buildme"

	"dev.hexasoftware.com/hxs/prettylog"
	git "gopkg.in/src-d/go-git.v4"
)

var (
	log = prettylog.New("GitFetcher")
)

func init() {
	// Register fetcher
	buildme.RegisterFetcher("git", &Git{})
}

type Git struct{}

// Fetcher return a project
func (g *Git) Fetch(fpath string) (*buildme.Project, error) {
	// File possibilities, does not cover the folder/subfolder
	if fpath == "." || fpath[0:2] == "./" || fpath[0] == '/' {
		absPath, err := filepath.Abs(fpath)
		if err != nil {
			return nil, err
		}
		fpath = "file://" + absPath
	}

	// Create tmpdir to clone project
	tmpdirname, err := ioutil.TempDir(buildme.TempDir(), "buildme.")
	if err != nil {
		return nil, err
	}
	_, err = git.PlainClone(tmpdirname, false, &git.CloneOptions{URL: fpath, Progress: os.Stdout})
	if err != nil {
		return nil, err
	}

	proj := buildme.NewProject()

	tarFile, err := ioutil.TempFile(buildme.TempDir(), "buildme.tar")
	if err != nil {
		return nil, err
	}

	err = tarutils.Tar(tmpdirname, tarFile, func(fl string, info os.FileInfo) bool {
		if info.Name() == "buildme.yml" {
			// Read all and add to conf
			data, err := ioutil.ReadFile(fl)
			if err != nil {
				return true
			}
			proj.ParseConf(data)
		}
		log.Println("Processing file:", fl, info)
		return true
	})
	if err != nil {
		return nil, err
	}
	tarFile.Close()

	os.RemoveAll(tmpdirname)
	proj.SetTarFile(tarFile.Name(), true) // Set as temporary will delete as soon as project closes

	return proj, nil
}
