package path

import (
	"buildme"
	"buildme/utils/tarutils"
	"errors"
	"io/ioutil"
	"os"
	"path/filepath"

	"dev.hexasoftware.com/hxs/prettylog"
)

var (
	log = prettylog.New("PathFetcher")
)

func init() {
	buildme.RegisterFetcher("path", &Path{})
}

type Path struct{}

func (p *Path) Fetch(fpath string) (*buildme.Project, error) {

	fpath, err := filepath.Abs(fpath)
	if err != nil {
		return nil, err
	}

	s, err := os.Stat(fpath)
	if err != nil {
		return nil, err
	}
	if !s.IsDir() {
		return nil, errors.New("Path should be a directory")
	}

	proj := buildme.NewProject()

	tarFile, err := ioutil.TempFile(buildme.TempDir(), "buildme.tar")
	if err != nil {
		return nil, err
	}

	err = tarutils.Tar(fpath, tarFile, func(fl string, info os.FileInfo) bool {
		if info.Name() == "buildme.yml" {
			// Read all and add to conf
			data, err := ioutil.ReadFile(fl)
			if err != nil {
				return true
			}
			proj.ParseConf(data)
		}

		return true
	})
	if err != nil {
		return nil, err
	}
	tarFile.Close()

	proj.SetTarFile(tarFile.Name(), true)

	return proj, nil

}
