package buildme

import (
	"fmt"
	"io"
	"os"

	yaml "gopkg.in/yaml.v2"
)

// Project type
type Project struct {
	tarFilename string
	isTemp      bool
	//utils.Filer
	//io.ReadCloser // Tar format? tar reader perhaps
	//Name       string
	//Dockerfile string

	Conf ProjectConf
	// Meta YML data
}

//NewProject Creates a project
func NewProject() *Project {
	return &Project{
		tarFilename: "",
		isTemp:      false,
		//	Name:       name,
		//Dockerfile: "docker/Dockerfile.build",
		Conf: ProjectConf{},
	}
}

func (p *Project) ParseConf(data []byte) error {
	return yaml.Unmarshal(data, &p.Conf)
}

//SetTarFile sets the source project tar
func (p *Project) SetTarFile(tarFile string, isTemp bool) *Project {
	p.tarFilename = tarFile
	p.isTemp = isTemp
	return p
}

func (p *Project) SetConf(projConf ProjectConf) *Project {
	p.Conf = projConf
	return p
}

func (p *Project) Reader() io.ReadCloser {
	f, _ := os.Open(p.tarFilename)
	return f
}

func (p *Project) Close() {
	if p.isTemp {
		log.Println("Closing temporary file")
		os.Remove(p.tarFilename)
	}
	//os.Remove(tarFile) // Should remove??
}

func (p *Project) Task(name string) error {

	log.Printf("Proj: %s Running task: %s", p.Conf.Project, name)
	log.Println("Services:", p.Conf.Services)

	taskconf, ok := p.Conf.Tasks[name]
	if !ok {
		return fmt.Errorf("Task '%s' not found\nTasks: %s", name, p.Conf.Tasks)
	}
	log.Println("Task found", taskconf)

	t := Task{name, taskconf, p, "", ""}

	if err := t.Exec(); err != nil {
		return err
	}
	// Start services if any

	// Task kind docker
	// Stupid docker cli

	return nil
}
