package tarutils

import (
	"archive/tar"
	"errors"
	"io"
	"os"
	"path/filepath"
	"strings"
)

//Tar tar a folder and return a reader?
func Tar(fpath string, w *os.File, fn func(fl string, info os.FileInfo) bool) error {
	var err error

	if !filepath.IsAbs(fpath) {
		return errors.New("Path must be absolute??")
	}
	// Add slash to path end so files can be relative
	if fpath[len(fpath)-1] != '/' {
		fpath += "/"
	}

	tw := tar.NewWriter(w)

	err = filepath.Walk(fpath, func(p string, info os.FileInfo, err error) error {
		if info.IsDir() && info.Name()[0] == '.' {
			return filepath.SkipDir
		}
		if info.IsDir() || info.Name()[0] == '.' { // Skip file/dir
			return nil
		}

		if fn(p, info) == false {
			return nil
		}

		var link string

		if info.Mode()&os.ModeSymlink == os.ModeSymlink {
			if link, err = os.Readlink(p); err != nil {
				return err
			}
		}

		npath := strings.TrimPrefix(p, fpath)
		theader, err := tar.FileInfoHeader(info, link)
		if err != nil {
			return err
		}
		theader.Name = npath // Relative path to root

		// Write file header
		err = tw.WriteHeader(theader)
		if err != nil {
			return err
		}

		if !info.Mode().IsRegular() {
			return nil
		}

		f, err := os.Open(p)
		if err != nil {
			return err
		}
		defer f.Close()

		_, err = io.Copy(tw, f)
		if err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return err
	}
	tw.Flush()

	return nil
}
